@@ -26,7 +26,7 @@ The basic components of this system are:
 """
 from __future__ import absolute_import
 __testify = 1
-__version__ = "0.5.3"
+__version__ = "0.5.4"
 
 import sys
 
@@ -52,8 +52,7 @@ from .test_fixtures import (
 
 from .utils import turtle
 
-from .test_program import TestProgram, main
-run = main
+from .test_program import TestProgram, main, run
 
 
 # We want default warning behavior for DeprecationWarning's thrown within testify.
