@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright 2009 Yelp
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,6 +17,7 @@ from __future__ import with_statement
 import contextlib
 from itertools import islice
 import re
+import warnings
 
 from .utils import stringdiffer
 
@@ -176,6 +178,20 @@ def assert_raises_such_that(exception_class, exception_test=lambda e: e, callabl
         with _assert_raises_context_manager(exception_class, exception_test):
             callable_obj(*args, **kwargs)
 
+
+def assert_raises_exactly(exception_class, *args):
+    """
+    Assert that a particular exception_class is raised with particular arguments.
+    Use this assertion when the exception message is important.
+    """
+    def test_exception(exception):
+        # We want to know the exact exception type, not that it has some superclass.
+        assert_is(type(exception), exception_class)
+        assert_equal(exception.args, args)
+
+    return assert_raises_such_that(exception_class, test_exception)
+
+
 def assert_raises_and_contains(expected_exception_class, strings, callable_obj, *args, **kwargs):
     """Assert an exception is raised by passing in a callable and its
     arguments and that the string representation of the exception
@@ -195,6 +211,7 @@ def assert_raises_and_contains(expected_exception_class, strings, callable_obj,
     else:
         assert_not_reached("No exception was raised (expected %s)" % expected_exception_class)
 
+
 @contextlib.contextmanager
 def _assert_raises_context_manager(exception_class, exception_test=lambda e: e):
     """Builds a context manager for testing that code raises an assertion.
@@ -232,6 +249,7 @@ def _diff_message(lhs, rhs):
     # Python2 exceptions require bytes.
     return message.encode('UTF-8')
 
+
 def assert_equal(lval, rval, message=None):
     """Assert that lval and rval are equal."""
     if message:
@@ -241,9 +259,30 @@ def assert_equal(lval, rval, message=None):
             "assertion failed: l == r\nl: %r\nr: %r\n\n%s" % \
                 (lval, rval, _diff_message(lval, rval))
 
+
 assert_equals = assert_equal
 
 
+def assert_true(lval, message=None):
+    """Assert that lval evaluates to True, not identity."""
+    if message:
+        assert bool(lval) == True, message
+    else:
+        assert bool(lval) == True, \
+            "assertion failed: l == r\nl: %r\nr: %r\n\n%s" % \
+                (lval, True, _diff_message(lval, True))
+
+
+def assert_false(lval, message=None):
+    """Assert that lval evaluates to False, not identity."""
+    if message:
+        assert bool(lval) == False, message
+    else:
+        assert bool(lval) == False, \
+            "assertion failed: l == r\nl: %r\nr: %r\n\n%s" % \
+                (lval, False, _diff_message(lval, False))
+
+
 def assert_almost_equal(lval, rval, digits, message=None):
     """Assert that lval and rval, when rounded to the specified number of digits, are the same."""
     real_message = message or "%r !~= %r" % (lval, rval)
@@ -312,14 +351,21 @@ def assert_between(a, b, c):
     assert_in_range(b, a, c, inclusive=True)
 
 
-def assert_in(item, sequence, msg="assertion failed: expected %(item)r in %(sequence)r"):
+def assert_in(item, sequence, message="assertion failed: expected %(item)r in %(sequence)r", msg=None):
     """Assert that the item is in the sequence."""
-    assert item in sequence, msg % {'item':item, 'sequence':sequence}
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
 
+    assert item in sequence, message % {'item':item, 'sequence':sequence}
 
-def assert_not_in(item, sequence, msg="assertion failed: expected %(item)r not in %(sequence)r"):
+def assert_not_in(item, sequence, message="assertion failed: expected %(item)r not in %(sequence)r", msg=None):
     """Assert that the item is not in the sequence."""
-    assert item not in sequence, msg % {'item':item, 'sequence':sequence}
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
+    assert item not in sequence, message % {'item':item, 'sequence':sequence}
 
 
 def assert_all_in(left, right):
@@ -333,14 +379,14 @@ def assert_all_in(left, right):
             unmatching.append(item)
     if unmatching:
         raise AssertionError(
-            'The following items were not found in not found in %s: %s' % (right, unmatching)
+            'The following items were not found in %s: %s' % (right, unmatching)
         )
 
 
 def assert_starts_with(val, prefix):
     """Assert that val.startswith(prefix)."""
-    msg = "%(val)r does not start with %(prefix)r" % locals()
-    assert val.startswith(prefix), msg
+    message = "%(val)r does not start with %(prefix)r" % locals()
+    assert val.startswith(prefix), message
 
 
 def assert_not_reached(message=None):
@@ -387,7 +433,8 @@ def assert_empty(iterable, max_elements_to_print=None, message=None):
         except TypeError:
             max_elements_to_print = 10
 
-    message = message or "iterable %s was unexpectedly non-empty." % iterable
+    # Build the message *before* touching iterable since that might modify it.
+    message = message or "iterable {0} was unexpectedly non-empty.".format(iterable)
 
     # Get the first max_elements_to_print + 1 items from iterable, or just
     # the first item if max_elements_to_print is 0.  Trying to get an
@@ -419,9 +466,10 @@ def assert_not_empty(iterable, message=None):
     for value in iterable:
         break
     else:
-        # the else clause of a for loop is reached iff you break out of the loop
+        # The else clause of a for loop is reached iff you break out of the loop.
         raise AssertionError(message if message else
-            "iterable %s is unexpectedly empty" % iterable)
+            "iterable {0} is unexpectedly empty".format(iterable)
+        )
 
 
 def assert_length(sequence, expected, message=None):
@@ -434,21 +482,29 @@ def assert_length(sequence, expected, message=None):
 def assert_call(turtle, call_idx, *args, **kwargs):
     """Assert that a function was called on turtle with the correct args."""
     actual = turtle.calls[call_idx] if turtle.calls else None
-    msg = "Call %s expected %s, was %s" % (call_idx, (args, kwargs), actual)
-    assert actual == (args, kwargs), msg
+    message = "Call %s expected %s, was %s" % (call_idx, (args, kwargs), actual)
+    assert actual == (args, kwargs), message
 
 
-def assert_is(left, right, msg="expected %(left)r is %(right)r"):
+def assert_is(left, right, message="expected %(left)r is %(right)r", msg=None):
     """Assert that left and right are the same object"""
-    assert left is right, msg % {'left':left, 'right': right}
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
+    assert left is right, message % {'left':left, 'right': right}
 
 
-def assert_is_not(left, right, msg="expected %(left)r is not %(right)r"):
+def assert_is_not(left, right, message="expected %(left)r is not %(right)r", msg=None):
     """Assert that left and right are NOT the same object"""
-    assert left is not right, msg % {'left':left, 'right':right}
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
+    assert left is not right, message % {'left':left, 'right':right}
 
 
-def assert_all_match_regex(pattern, values, msg="expected %(value)r to match %(pattern)r"):
+def assert_all_match_regex(pattern, values, message="expected %(value)r to match %(pattern)r", msg=None):
     """Assert that all values in an iterable match a regex pattern.
 
     Args:
@@ -458,8 +514,12 @@ def assert_all_match_regex(pattern, values, msg="expected %(value)r to match %(p
     Raises AssertionError if any value does not match.
 
     """
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
     for value in values:
-        assert re.match(pattern, value), msg % {'value':value, 'pattern':pattern}
+        assert re.match(pattern, value), message % {'value':value, 'pattern':pattern}
 
 
 def assert_match_regex(pattern, value, *args, **kwargs):
@@ -467,7 +527,7 @@ def assert_match_regex(pattern, value, *args, **kwargs):
     assert_all_match_regex(pattern, [value], *args, **kwargs)
 
 
-def assert_any_match_regex(pattern, values, msg="expected at least one %(values)r to match %(pattern)r"):
+def assert_any_match_regex(pattern, values, message="expected at least one %(values)r to match %(pattern)r", msg=None):
     """Assert that at least one value in an iterable matches a regex pattern.
 
     Args:
@@ -477,14 +537,18 @@ def assert_any_match_regex(pattern, values, msg="expected at least one %(values)
     Raises AssertionError if all values don't match.
 
     """
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
     for value in values:
         if re.match(pattern, value) is not None:
             return
 
-    raise AssertionError(msg % {'values':values, 'pattern':pattern})
+    raise AssertionError(message % {'values':values, 'pattern':pattern})
 
 
-def assert_all_not_match_regex(pattern, values, msg="expected %(value)r to not match %(pattern)r"):
+def assert_all_not_match_regex(pattern, values, message="expected %(value)r to not match %(pattern)r", msg=None):
     """Assert that all values don't match a regex pattern.
 
     Args:
@@ -494,16 +558,24 @@ def assert_all_not_match_regex(pattern, values, msg="expected %(value)r to not m
     Raises AssertionError if any values matches.
 
     """
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
     for value in values:
-        assert not re.match(pattern, value), msg % {'value':value, 'pattern':pattern}
+        assert not re.match(pattern, value), message % {'value':value, 'pattern':pattern}
 
 
-def assert_sets_equal(left, right, msg="expected %(left)r == %(right)r [left has:%(extra_left)r, right has:%(extra_right)r]"):
+def assert_sets_equal(left, right, message="expected %(left)r == %(right)r [left has:%(extra_left)r, right has:%(extra_right)r]", msg=None):
     """Assert that two sets are equal."""
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
     if left != right:
         extra_left = left - right
         extra_right = right - left
-        raise AssertionError(msg % {
+        raise AssertionError(message % {
             'left': left,
             'right': right,
             'extra_left': extra_left,
@@ -511,8 +583,12 @@ def assert_sets_equal(left, right, msg="expected %(left)r == %(right)r [left has
         })
 
 
-def assert_dicts_equal(left, right, ignore_keys=None, msg="expected %(left)r == %(right)r [left has:%(extra_left)r, right has:%(extra_right)r]"):
+def assert_dicts_equal(left, right, ignore_keys=None, message="expected %(left)r == %(right)r [left has:%(extra_left)r, right has:%(extra_right)r]", msg=None):
     """Assert that two dictionarys are equal (optionally ignoring certain keys)."""
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
     if ignore_keys is not None:
         left = dict((k, left[k]) for k in left if k not in ignore_keys)
         right = dict((k, right[k]) for k in right if k not in ignore_keys)
@@ -522,7 +598,7 @@ def assert_dicts_equal(left, right, ignore_keys=None, msg="expected %(left)r ==
 
     extra_left = _dict_subtract(left, right)
     extra_right = _dict_subtract(right, left)
-    raise AssertionError(msg % {
+    raise AssertionError(message % {
         'left': left,
         'right': right,
         'extra_left': extra_left,
@@ -530,8 +606,12 @@ def assert_dicts_equal(left, right, ignore_keys=None, msg="expected %(left)r ==
     })
 
 
-def assert_dict_subset(left, right, msg="expected [subset has:%(extra_left)r, superset has:%(extra_right)s]"):
+def assert_dict_subset(left, right, message="expected [subset has:%(extra_left)r, superset has:%(extra_right)s]", msg=None):
     """Assert that a dictionary is a strict subset of another dictionary (both keys and values)."""
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
     difference_dict = _dict_subtract(left, right)
 
     if not difference_dict:
@@ -540,7 +620,7 @@ def assert_dict_subset(left, right, msg="expected [subset has:%(extra_left)r, su
     extra_left = difference_dict
     small_right = dict((k, right[k]) for k in right if k in left.keys())
     extra_right = _dict_subtract(small_right, left)
-    raise AssertionError(msg % {
+    raise AssertionError(message % {
         'left': left,
         'right': right,
         'extra_left': extra_left,
@@ -548,13 +628,17 @@ def assert_dict_subset(left, right, msg="expected [subset has:%(extra_left)r, su
     })
 
 
-def assert_subset(left, right, msg="expected %(set_left)r <= %(set_right)r [left has:%(extra)r]"):
+def assert_subset(left, right, message="expected %(set_left)r <= %(set_right)r [left has:%(extra)r]", msg=None):
     """Assert that the left set is a subset of the right set."""
+    if msg:
+        warnings.warn("msg is deprecated", DeprecationWarning)
+        message = msg
+
     set_left = set(left)
     set_right = set(right)
     if not (set_left <= set_right):
         extra = set_left - set_right
-        raise AssertionError(msg % {
+        raise AssertionError(message % {
             'left': left,
             'right': right,
             'set_left': set_left,
@@ -591,11 +675,15 @@ def assert_exactly_one(*args, **kwargs):
         True if desired conditions are satisfied. For example:
 
         >>> assert_exactly_one(True, False, truthy_fxn=bool) # Success
+        True
 
         >>> assert_exactly_one(0, None) # Success
+        0
 
         >>> assert_exactly_one(True, False)
-        AssertionError
+        Traceback (most recent call last):
+            ...
+        AssertionError: Expected exactly one True (got 2) args: (True, False)
 
     Returns:
         The argument that passes the truthy function
@@ -610,6 +698,85 @@ def assert_exactly_one(*args, **kwargs):
     return true_args[0]
 
 
+@contextlib.contextmanager
+def _assert_warns_context_manager(warning_class=None, warnings_test=None):
+    """
+    Builds a context manager for testing code that should throw a warning.
+    This will look for a given class, call a custom test, or both.
+
+    Args:
+        warning_class - a class or subclass of Warning. If not None, then
+            the context manager will raise an AssertionError if the block
+            does not throw at least one warning of that type.
+        warnings_test - a function which takes a list of warnings caught,
+            and makes a number of assertions about the result. If the function
+            returns without an exception, the context manager will consider
+            this a successful assertion.
+    """
+    with warnings.catch_warnings(record=True) as caught:
+        # All warnings should be triggered.
+        warnings.resetwarnings()
+        if warning_class:
+            warnings.simplefilter('ignore')
+            warnings.simplefilter('always', category=warning_class)
+        else:
+            warnings.simplefilter('always')
+        # Do something that ought to trigger a warning.
+        yield
+        # We should have received at least one warning.
+        assert_gt(len(caught), 0, 'expected at least one warning to be thrown')
+        # Run the custom test against the warnings we caught.
+        if warnings_test:
+            warnings_test(caught)
+
+
+def assert_warns(warning_class=None, callable=None, *args, **kwargs):
+    """Assert that the given warning class is thrown as a context manager
+    or by passing in a callable and its arguments.
+
+    As a context manager:
+    >>> with assert_warns():
+    ...     warnings.warn('Hey!')
+
+    Passing in a callable:
+    >>> def throw_warning():
+    ...     warnings.warn('Hey!')
+    >>> assert_warns(UserWarning, throw_warning)
+    """
+    if callable is None:
+        return _assert_warns_context_manager(warning_class=warning_class)
+    else:
+        with _assert_warns_context_manager(warning_class=warning_class):
+            callable(*args, **kwargs)
+
+
+def assert_warns_such_that(warnings_test, callable=None, *args, **kwargs):
+    """
+    Assert that the given warnings_test function returns True when
+    called with a full list of warnings that were generated by either
+    a code block (when this is used as a context manager in a `with` statement)
+    or the given callable (when called with the appropriate args and kwargs).
+
+    As a context manager:
+    >>> def two_warnings_thrown(warnings):
+    ...     assert len(warnings) == 2
+    >>> with assert_warns_such_that(two_warnings_thrown):
+    ...     warnings.warn('Hey!')
+    ...     warnings.warn('Seriously!')
+
+    Passing in a callable:
+    >>> def throw_warnings(count):
+    ...     for n in range(count):
+    ...         warnings.warn('Warning #%i' % n)
+    >>> assert_warns_such_that(two_warnings_thrown, throw_warnings, 2)
+    """
+    if callable is None:
+        return _assert_warns_context_manager(warnings_test=warnings_test)
+    else:
+        with _assert_warns_context_manager(warnings_test=warnings_test):
+            callable(*args, **kwargs)
+
+
 def _to_characters(x):
     """Return characters that represent the object `x`, come hell or high water."""
     if isinstance(x, unicode):
